@@ -1,9 +1,9 @@
 /*
  * PDFedit - free program for PDF document manipulation.
- * Copyright (C) 2006, 2007, 2008  PDFedit team: Michal Hocko,
- *                                              Miroslav Jahoda,
- *                                              Jozef Misutka,
- *                                              Martin Petricek
+ * Copyright (C) 2006-2009  PDFedit team: Michal Hocko,
+ *                                        Jozef Misutka,
+ *                                        Martin Petricek
+ *                   Former team members: Miroslav Jahoda
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,6 +30,9 @@
 #include <qstringlist.h>
 #include <qvariant.h>
 
+namespace pdfobjects {
+ class CPdf;
+}
 namespace gui {
 
 class QSAnnotation;
@@ -99,7 +102,7 @@ public slots: //This will be all exported to scripting
  /*- Creates and returns new IProperty of type String */
  QSIProperty* createString(const QString &value);
  /*-
-  Tries to delinearize PDF, reading from input file and writing delinearized result to output file.
+  Tries to delinearize the PDF, reading from input file and writing delinearized result to output file.
   Does not check for overwriting output.
   Returns true if delinearization was successful, false in case of failure.
   In case of failure the error mesage is available via error() function.
@@ -112,6 +115,13 @@ public slots: //This will be all exported to scripting
  QString error();
  /*- Returns true if given file exists on disk, false otherwise */
  bool exists(const QString &chkFileName);
+ /*-
+  Tries to flatten the PDF, reading from input file and writing flattened result to output file.
+  Does not check for overwriting output.
+  Returns true if flattening was successful, false in case of failure.
+  In case of failure the error mesage is available via error() function.
+ */
+ bool flatten(const QString &inFile,const QString &outFile);
  /*-
   Debugging function usable by script developers.
   Returns list of all functions that are present in current script interpreter.
@@ -129,8 +139,11 @@ public slots: //This will be all exported to scripting
   after it don't need to use it anymore.
   If advancedMode is set to true, document is opened in "advanced mode"
   (more advanced, but also more dangerous changes to it are possible)
+  If askPassword is set to false, password to document (if the document is encrypted)
+  will not be solicited from user and setPassword method of PDF object have to be
+  called in script with correct password to allow manipulating the encrypted document.
  */
- QSPdf* loadPdf(const QString &name,bool advancedMode=false);
+ QSPdf* loadPdf(const QString &name,bool advancedMode=false, bool askPassword=true);
  /*-
   Debugging function usable by script developers.
   Returns list of all objects that are in current script interpreter
@@ -249,6 +262,9 @@ public slots: //This will be all exported to scripting
  */
  QString version();
 
+protected:
+ virtual boost::shared_ptr<pdfobjects::CPdf> getBasePdfInstance(const QString &filename, const QString &openMode=QString::null, bool askPassword=true);
+
 private:
  bool runFile(const QString &scriptName);
 protected:
