@@ -617,7 +617,7 @@
     </message>
     <message>
         <source>Saving as</source>
-        <translation>Сохраняется как</translation>
+        <translation type="obsolete">Сохраняется как</translation>
     </message>
     <message>
         <source>Warning: This document is linearized PDF!</source>
@@ -1076,6 +1076,30 @@ Data: %3
         <source>Use &apos;%1 --help&apos; to see list of commandline options and their parameters</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Saved as</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Unable to open file: %1</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Not implemented: %1</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Unknown error</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Apply from page</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>How many pages</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>annotation_type</name>
@@ -1796,7 +1820,7 @@ Data: %3
     </message>
     <message>
         <source>Error occured while saving copy</source>
-        <translation>При сохранении копии произошла неизвестная ошибка</translation>
+        <translation type="obsolete">При сохранении копии произошла неизвестная ошибка</translation>
     </message>
     <message>
         <source>viewing revision</source>
@@ -1822,6 +1846,16 @@ Data: %3
         <source>Loaded file</source>
         <translation>Загружен файл</translation>
     </message>
+    <message>
+        <source>Saving not implemented:
+%1</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Error occured while saving copy:
+%1</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>gui::Property</name>
@@ -1920,6 +1954,11 @@ Data: %3
         <source>Page not found</source>
         <translation>Страница не найдена</translation>
     </message>
+    <message>
+        <source>Unable to save:
+%1</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>gui::RefPropertyDialog</name>
