@@ -311,10 +311,6 @@ Data: %3
         <source>Changed revision to:</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>Saving as</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Operators</source>
         <translation type="unfinished"></translation>
@@ -1076,6 +1072,30 @@ Data: %3
         <source>Use &apos;%1 --help&apos; to see list of commandline options and their parameters</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Saved as</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Unable to open file: %1</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Not implemented: %1</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Unknown error</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Apply from page</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>How many pages</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>annotation_type</name>
@@ -1798,10 +1818,6 @@ Data: %3
         <source>Unknown error occured while saving document</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>Error occured while saving copy</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Linearized PDF</source>
         <translation type="unfinished"></translation>
@@ -1822,6 +1838,16 @@ Data: %3
         <source>Selected annotations</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Saving not implemented:
+%1</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Error occured while saving copy:
+%1</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>gui::Property</name>
@@ -1920,6 +1946,11 @@ Data: %3
         <source>Unknown error occured while saving document</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Unable to save:
+%1</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>gui::RefPropertyDialog</name>
