@@ -1,9 +1,9 @@
 /*
  * PDFedit - free program for PDF document manipulation.
- * Copyright (C) 2006, 2007, 2008  PDFedit team: Michal Hocko,
- *                                              Miroslav Jahoda,
- *                                              Jozef Misutka,
- *                                              Martin Petricek
+ * Copyright (C) 2006-2009  PDFedit team: Michal Hocko,
+ *                                        Jozef Misutka,
+ *                                        Martin Petricek
+ *                   Former team members: Miroslav Jahoda
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,8 +32,9 @@
 #include "kernel/pdfoperators.h"
 #include "kernel/stateupdater.h"
 #include "kernel/cobject.h"
-#include "kernel/factories.h"
 #include "kernel/cinlineimage.h"
+#include "kernel/contentschangetag.h"
+#include "kernel/pdfoperatorsiter.h"
 
 //fabs
 #include <math.h>
@@ -67,10 +68,10 @@ namespace {
 	 */
 	void
 	opsSetPdfRefCs (shared_ptr<PdfOperator> first, 
-					CPdf& pdf, 
+					boost::weak_ptr<CPdf> pdf, 
 					IndiRef rf, 
 					CContentStream& cs, 
-					boost::shared_ptr<IIPropertyObserver> observer)
+					boost::shared_ptr<IPropertyObserver> observer)
 	{
 		utilsPrintDbg (DBG_DBG, "");
 		CContentStream::OperatorIterator it = PdfOperator::getIterator (first);
@@ -84,74 +85,12 @@ namespace {
 			// 	-- cstream won't get notified by its children, not needed
 			// 
 			if (0 < it.getCurrent()->getParametersCount())
-				it.getCurrent()->init_operands (observer, &pdf, &rf);
+				it.getCurrent()->init_operands (observer, pdf, &rf);
 			
 			it = it.next ();
 		}
 	}
 	
-	/**
-	 * Check if the operands match the specification and replace operand with
-	 * its stronger equivalent.
-	 *
-	 * (e.g. When xpdf returns an object with integer type, but the operand can be a real, we have to
-	 * convert it to real.)
-	 *
-	 * @param ops Operator specification
-	 * @param operands Operand stack.
-	 *
-	 * @return True if type and count match, false otherwise.
-	 */
-	bool checkAndFix (const StateUpdater::CheckTypes& ops, PdfOperator::Operands& operands)
-	{
-		size_t argNum = static_cast<size_t> ((ops.argNum > 0) ? ops.argNum : -ops.argNum);
-			
-		//
-		// Check operator size if > 0 than it is the exact size, maximum
-		// otherwise
-		//
-		if (((ops.argNum >= 0) && (operands.size() != argNum)) 
-			 || ((ops.argNum <  0) && (operands.size() > argNum)) )
-		{
-			utilsPrintDbg (DBG_ERR, "Number of operands mismatch.. expected " << ops.argNum << " got: " << operands.size());
-			return false;
-		}
-		
-		//
-		// Check arguments
-		//
-		PdfOperator::Operands::reverse_iterator rit = operands.rbegin ();
-		// Be careful -- buffer overflow
-		argNum = std::min (argNum, operands.size());
-		advance (rit, argNum);
-		PdfOperator::Operands::iterator it = rit.base ();
-		// Loop from the first operator to the end
-		for (int pos = 0; it != operands.end (); ++it, ++pos)
-		{			
-  			if (!isBitSet(ops.types[pos], (*it)->getType()))
-			{
-				utilsPrintDbg (DBG_ERR, "Bad " << pos << "-th operand type [" << (*it)->getType() << "] " << hex << " 0x" << ops.types[pos]);
-				return false;
-			}
-
-			// 
-			// If xpdf returned an Int, but the operand can be a real convert it
-			// 
-			if (isInt(*it))
-			{
-  				if (isBitSet(ops.types[pos], pReal))
-				{ // Convert it to real
-					double dval = 0.0;
-					dval = IProperty::getSmartCObjectPtr<CInt>(*it)->getValue();
-					shared_ptr<IProperty> pIp (new CReal (dval));
-					std::replace (operands.begin(), operands.end(), *it, pIp);
-				}
-			}
-		}
-
-		return true;
-	}
-
 	
 	/**
 	 * Parse inline image. 
@@ -203,7 +142,7 @@ namespace {
 		// Bad content stream
 		if (streamreader.eof())
 		{
-			utilsPrintDbg (debug::DBG_CRIT, "Content stream is damaged...");
+			utilsPrintDbg (debug::DBG_ERR, "Content stream is damaged...");
 			return NULL;
 		}
 	
@@ -247,7 +186,7 @@ namespace {
 	 * @return True if everything ok, false if end of stream reached.
 	 */
 	bool
-	createOperands (CStreamsXpdfReader<CContentStream::CStreams>& streamreader, 
+	createOperandsFromStream (CStreamsXpdfReader<CContentStream::CStreams>& streamreader, 
 					PdfOperator::Operands& operands,
 					xpdf::XpdfObject& o)
 	{
@@ -286,54 +225,35 @@ namespace {
 	 * @param operands Operands of operator. They are shared through subcalls.
 	 */
 	shared_ptr<PdfOperator>
-	createOperator (CStreamsXpdfReader<CContentStream::CStreams>& streamreader, 
+	createOperatorFromStream (CStreamsXpdfReader<CContentStream::CStreams>& streamreader, 
 					PdfOperator::Operands& operands)
 	{
 		// Get operands
 		xpdf::XpdfObject o;
-		if (!createOperands (streamreader, operands, o))
+		if (!createOperandsFromStream (streamreader, operands, o))
 			return shared_ptr<PdfOperator> ();
 		
-		// Try to find the op by its name
-		const StateUpdater::CheckTypes* chcktp = StateUpdater::findOp (o->getCmd());
-		// Operator not found, create unknown operator
-		if (NULL == chcktp)
-			return shared_ptr<PdfOperator> (new SimpleGenericOperator (string (o->getCmd()),operands));
-		
-		assert (chcktp);
-		utilsPrintDbg (DBG_DBG, "Operator found. " << chcktp->name);
-
-		//
-		// Check the type against specification
-		// 
-		if (!checkAndFix (*chcktp, operands))
-		{
-			//assert (!"Content stream bad operator type.");
-			throw ElementBadTypeException ("Content stream operator has incorrect operand type.");
-		}
-
 		//
 		// SPECIAL CASE for inline image (stream within a text stream)
 		//
-		if ( 0 == strncmp (chcktp->name, "BI", 2))
+		if ( 0 == strncmp (o->getCmd(), "BI", 2))
 		{
 			utilsPrintDbg (debug::DBG_DBG, "");
+			const StateUpdater::CheckTypes* chcktp = StateUpdater::findOp (o->getCmd());
+			assert(chcktp);
+			if (!checkAndFixOperator (*chcktp, operands))
+			{
+				//assert (!"Content stream bad operator type.");
+				throw ElementBadTypeException ("Content stream operator has incorrect operand type.");
+			}
 			
 			shared_ptr<CInlineImage> inimg (getInlineImage (streamreader));
 			return shared_ptr<PdfOperator> (new InlineImageCompositePdfOperator (chcktp->name, chcktp->endTag, inimg));
 		}
-		
-		// Get operands count
-		size_t argNum = static_cast<size_t> ((chcktp->argNum > 0) ? chcktp->argNum : -chcktp->argNum);
 
-		//
-		// If endTag is "" it is a simple operator, composite otherwise
-		// 
-		if (isSimpleOp(*chcktp))
-			return shared_ptr<PdfOperator> (new SimpleGenericOperator (chcktp->name, argNum, operands));
-			
-		else // Composite operator
-			return shared_ptr<PdfOperator> (new UnknownCompositePdfOperator (chcktp->name, chcktp->endTag));	
+		// factory function for all other operators
+		std::string name = o->getCmd();
+		return createOperator(name, operands);
 	}
 	
 	/**
@@ -354,7 +274,7 @@ namespace {
 	parseOp (CStreamsXpdfReader<CContentStream::CStreams>& streamreader, PdfOperator::Operands& operands)
 	{
 		// Create operator with its operands
-		shared_ptr<PdfOperator> result = createOperator (streamreader, operands);
+		shared_ptr<PdfOperator> result = createOperatorFromStream (streamreader, operands);
 	
 		if (result && isCompositeOp (result) && !isInlineImageOp (result))
 		{
@@ -363,7 +283,7 @@ namespace {
 			string endtag = StateUpdater::getEndTag (opname);
 			bool foundEndTag = false;
 			
-			// The same as in (re)parseContentStream
+			// The same as in (re)parse
 			shared_ptr<PdfOperator> newop, previousLast = result;
 
 			//
@@ -413,10 +333,10 @@ namespace {
 	 * @param parsedstreams Streams that have been really parsed.
 	 */
 	void
-	parseContentStream (CContentStream::Operators& operators, 
+	parse (CContentStream::Operators& operators, 
 						CContentStream::CStreams& streams, 
 						CContentStream& cs,
-						boost::shared_ptr<IIPropertyObserver> observer,
+						boost::shared_ptr<IPropertyObserver> observer,
 						CContentStream::CStreams* parsedstreams = NULL)
 	{
 		// Clear operators
@@ -429,8 +349,8 @@ namespace {
 			if (!hasValidPdf (*it) || !hasValidRef (*it))
 				throw CObjInvalidObject ();
 		}
-		CPdf* pdf = streams.front()->getPdf ();
-		assert (pdf);
+		weak_ptr<CPdf> pdf = streams.front()->getPdf ();
+		assert (pdf.lock());
 		IndiRef rf = streams.front()->getIndiRef ();
 
 		assert (!streams.empty());
@@ -453,14 +373,14 @@ namespace {
 				//
 				// Is it our change
 				//
-				if (isPdfOp (*newop,getChangeTagName()))
+				if (isPdfOp (*newop,ContentsChangeTag::CHANGE_TAG_NAME))
 				{	
 					PdfOperator::Operands ops;
 					newop->getParameters (ops);
 					if (!ops.empty())
 					{
 						try {
-							if (getChangeTagId() == getNameFromIProperty(ops.front()))
+							if (ContentsChangeTag::CHANGE_TAG_ID == getNameFromIProperty(ops.front()))
 								our_change = true;
 						}catch (ElementBadTypeException&)
 							{}
@@ -487,7 +407,7 @@ namespace {
 							break;
 						xpdf::XpdfObject o;
 						streamreader.lookXpdfObject (*o);
-						if (o->isName (getChangeTagId().c_str()))
+						if (o->isName (ContentsChangeTag::CHANGE_TAG_ID))
 							break;
 					}
 				}
@@ -530,11 +450,11 @@ namespace {
 			}
 		}
 
-		assert (operands.empty());
-		assert (observer);
+			assert (operands.empty());
+			assert (observer);
 		// Set pdf ref and cs
 		if (!operators.empty())
-			opsSetPdfRefCs (operators.front(), *pdf, rf, cs, observer);
+			opsSetPdfRefCs (operators.front(), pdf, rf, cs, observer);
 	}
 
 
@@ -615,6 +535,9 @@ CContentStream::OperandObserver::notify (boost::shared_ptr<IProperty> newValue,
 										  boost::shared_ptr<const IProperty::ObserverContext>) const 
 throw ()
 {
+		if (_locked) 
+			return;
+
 	try {
 
 		utilsPrintDbg (debug::DBG_DBG, "");
@@ -666,8 +589,8 @@ CContentStream::CContentStream (CStreams& strs,
 	}
 	
 	// Parse it into small objects
-	assert (gfxres);
-	assert (gfxstate);
+		assert (gfxres);
+		assert (gfxstate);
 	
 	// Create cstream observer and register it on all operands
 	cstreamobserver = boost::shared_ptr<CStreamObserver> (new CStreamObserver (this));
@@ -675,7 +598,7 @@ CContentStream::CContentStream (CStreams& strs,
 	operandobserver = boost::shared_ptr<OperandObserver> (new OperandObserver (this));
 	
 	// Parse it, move parsed straems from strs to cstreams
-	parseContentStream (operators, strs, *this, operandobserver, &cstreams);
+	parse (operators, strs, *this, operandobserver, &cstreams);
 	
 	// Save bounding boxes
 	if (!operators.empty()) 
@@ -710,7 +633,7 @@ CContentStream::reparse (bool bboxOnly, boost::shared_ptr<GfxState> state, boost
 	{
 		// Clear operators	
 		operators.clear ();
-		parseContentStream (operators, cstreams, *this, operandobserver);
+		parse (operators, cstreams, *this, operandobserver);
 	}
 	
 	// Save bounding boxes
@@ -718,6 +641,43 @@ CContentStream::reparse (bool bboxOnly, boost::shared_ptr<GfxState> state, boost
 		StateUpdater::updatePdfOperators (PdfOperator::getIterator (operators.front()), gfxres, *gfxstate, BBoxUpdater());
 }
 
+//
+//
+//
+void 
+CContentStream::replaceText (const std::string& what, const std::string& with)
+{
+	bool dirty = false;
+		if (operators.empty())
+			return;
+
+	operandobserver->lock();
+
+	TextOperatorIterator tit = PdfOperator::getIterator<TextOperatorIterator> (operators.front());
+	while (!tit.isEnd())
+	{
+		// uff
+		boost::shared_ptr<TextSimpleOperator> _cur 
+				= boost::dynamic_pointer_cast<TextSimpleOperator, PdfOperator> (tit.getCurrent());
+		std::string tmp;
+		_cur->getRawText (tmp);
+		string replaced = boost::replace_all_copy (tmp, what, with);
+		if (tmp != replaced)
+		{
+			dirty = true;
+			boost::shared_ptr<TextSimpleOperator> _cur 
+					= boost::dynamic_pointer_cast<TextSimpleOperator, PdfOperator> (tit.getCurrent());
+			_cur->setRawText (replaced);
+		}
+		tit.next();
+	}
+
+	operandobserver->unlock();
+	if (dirty)
+		_objectChanged();
+}
+
+
 //
 //
 //
@@ -897,10 +857,10 @@ CContentStream::insertOperator (OperatorIterator it, boost::shared_ptr<PdfOperat
 	// Set correct IndiRef, CPdf and cs to inserted operator
 	assert (hasValidRef (cstreams.front()));
 	assert (hasValidPdf (cstreams.front()));
-	CPdf* pdf = cstreams.front()->getPdf();
-	assert (pdf);
+	weak_ptr<CPdf> pdf = cstreams.front()->getPdf();
+	assert (pdf.lock());
 	IndiRef rf = cstreams.front()->getIndiRef ();
-	opsSetPdfRefCs (newOper, *pdf, rf, *this, operandobserver);
+	opsSetPdfRefCs (newOper, pdf, rf, *this, operandobserver);
 
 	//
 	// Insert into operators or composite
@@ -978,17 +938,17 @@ CContentStream::frontInsertOperator (boost::shared_ptr<PdfOperator> newoper,
 	// Check whether we can make the change
 	cstreams.front()->canChange();
 	IndiRef rf = cstreams.front()->getIndiRef ();
-	CPdf* pdf = cstreams.front()->getPdf();
-		assert (pdf);
+	weak_ptr<CPdf> pdf = cstreams.front()->getPdf();
+	assert (pdf.lock());
 	// set accordingly	
-	opsSetPdfRefCs (newoper, *pdf, rf, *this, operandobserver);
+	opsSetPdfRefCs (newoper, pdf, rf, *this, operandobserver);
 
 	if (operators.empty ())
 	{ // Insert into empty contentstream
 		operators.push_back (newoper);
 	}else
 	{ // Insert into
-		opsSetPdfRefCs (newoper, *pdf, rf, *this, operandobserver);
+		opsSetPdfRefCs (newoper, pdf, rf, *this, operandobserver);
 
 		shared_ptr<PdfOperator> secondoper = operators.front();
 		operators.push_front (newoper);
@@ -1033,10 +993,10 @@ CContentStream::replaceOperator (OperatorIterator it,
 	// Set correct IndiRef, CPdf and cs to inserted operator
 	assert (hasValidRef (cstreams.front()));
 	assert (hasValidPdf (cstreams.front()));
-	CPdf* pdf = cstreams.front()->getPdf();
-	assert (pdf);
+	weak_ptr<CPdf> pdf = cstreams.front()->getPdf();
+	assert (pdf.lock());
 	IndiRef rf = cstreams.front()->getIndiRef ();
-	opsSetPdfRefCs (newOper, *pdf, rf, *this, operandobserver);
+	opsSetPdfRefCs (newOper, pdf, rf, *this, operandobserver);
 
 	//
 	// Replace in operators or composite
@@ -1155,6 +1115,8 @@ CContentStream::unregisterCStreamObservers () const
 }
 
 
+
+
 //==========================================================
 // Operator helper functions
 //==========================================================
