@@ -1,9 +1,9 @@
 /*
  * PDFedit - free program for PDF document manipulation.
- * Copyright (C) 2006, 2007, 2008  PDFedit team: Michal Hocko,
- *                                              Miroslav Jahoda,
- *                                              Jozef Misutka,
- *                                              Martin Petricek
+ * Copyright (C) 2006-2009  PDFedit team: Michal Hocko,
+ *                                        Jozef Misutka,
+ *                                        Martin Petricek
+ *                   Former team members: Miroslav Jahoda
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -130,7 +130,7 @@ public:
 	 * @param o Xpdf object. 
 	 * @param rf Indirect id and gen id.
 	 */
-	CObjectSimple (CPdf& p, Object& o, const IndiRef& rf);
+	CObjectSimple (boost::weak_ptr<CPdf> p, Object& o, const IndiRef& rf);
 	
 	/**
 	 * Constructor.
@@ -328,9 +328,10 @@ makeNamePdfValid (Iter it, Iter end)
 	{
 		if ( '!' > (*it) || '~' < (*it))
 		{ // Convert it to ascii
-			Item a = (Item) (*it) >> sizeof (Item) * 4;
-			Item b = (Item) (*it) & ((unsigned) 1 << sizeof(Item) * 4);
-			tmp += std::string ("#") + a + b;
+			char hexstr[4];
+			snprintf(hexstr, sizeof(hexstr), "#%02x", (unsigned)(*it)&0xff);
+			assert(hexstr[sizeof(hexstr)-1] == '\0');
+			tmp += hexstr;
 		}else
 			tmp += *it;
 	}
@@ -420,7 +421,7 @@ makeStringPdfValid (GString * str)
  *
  * @return Pointer to newly created object.
  */
-IProperty* createObjFromXpdfObj (CPdf& pdf, ::Object& obj,const IndiRef& ref);
+IProperty* createObjFromXpdfObj (boost::shared_ptr<CPdf> pdf, ::Object& obj,const IndiRef& ref);
 
 /**
  * Creates CObject* from xpdf object.
@@ -570,7 +571,7 @@ boost::shared_ptr<IProperty> getReferencedObject (boost::shared_ptr<IProperty> i
  */
 template<typename ItemType, PropertyType ItemPType, typename Value>
 inline Value
-getValueFromSimple (const boost::shared_ptr<IProperty>& ip) __attribute__((deprecated));
+getValueFromSimple (const boost::shared_ptr<IProperty>& ip) DEPRECATED;
 
 // function definition (gcc doesn't like __attribute__ in function definition
 template<typename ItemType, PropertyType ItemPType, typename Value>
