@@ -1,9 +1,9 @@
 /*
  * PDFedit - free program for PDF document manipulation.
- * Copyright (C) 2006, 2007, 2008  PDFedit team: Michal Hocko,
- *                                              Miroslav Jahoda,
- *                                              Jozef Misutka,
- *                                              Martin Petricek
+ * Copyright (C) 2006-2009  PDFedit team: Michal Hocko,
+ *                                        Jozef Misutka,
+ *                                        Martin Petricek
+ *                   Former team members: Miroslav Jahoda
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,11 +27,8 @@
 
 // all basic includes
 #include "kernel/static.h"
-#include "kernel/iproperty.h"
-// Filters
 #include "kernel/cstream.h"
 #include "kernel/cobjecthelpers.h"
-#include "kernel/xpdf.h"
 
 /** 
  * Content streams can be separated in crazy
@@ -148,11 +145,17 @@ public:
 		close();
 	}
 
-	/** Get xpdf object. */
+	/** Get xpdf object.
+	 * @throw MalformedFormatExeption if not able to parse object.
+	 */
 	void getXpdfObject (::Object& obj)
 	{
 		curobj.reset ();
-		parser->getObj (curobj.get());
+		if(!parser->getObj (curobj.get()))
+		{
+			kernelPrintDbg(debug::DBG_ERR, "Unabel to parse object");
+			throw MalformedFormatExeption("content stream parse");
+		}
 		curobj->copy (&obj);
 	}
 
