@@ -1,9 +1,9 @@
 /*
  * PDFedit - free program for PDF document manipulation.
- * Copyright (C) 2006, 2007, 2008  PDFedit team: Michal Hocko,
- *                                              Miroslav Jahoda,
- *                                              Jozef Misutka,
- *                                              Martin Petricek
+ * Copyright (C) 2006-2009  PDFedit team: Michal Hocko,
+ *                                        Jozef Misutka,
+ *                                        Martin Petricek
+ *                   Former team members: Miroslav Jahoda
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,11 +23,33 @@
  */
 // vim:tabstop=4:shiftwidth=4:noexpandtab:textwidth=80
 #include "kernel/static.h"
+#include "xpdf/Object.h"
+#include "xpdf/encrypt_utils.h"
 #include "kernel/cxref.h"
 #include "utils/debug.h"
 #include "kernel/factories.h"
+#include "kernel/pdfedit-core-dev.h"
 
 using namespace pdfobjects;
+CXref::CXref(BaseStream * stream):XRef(stream), internal_fetch(true)
+{
+	if(!pdfedit_core_dev_init_check())
+	{
+		kernelPrintDbg(debug::DBG_CRIT, "pdfedit-core-dev core is not initialized");
+		throw PDFedit_devException();
+	}
+	if(!isOk())
+	{
+		// xref is corrupted
+		kernelPrintDbg(debug::DBG_ERR, 
+				"Unable to parse xref table. errorCode="
+				<<getErrorCode());
+		throw MalformedFormatExeption("XRef parsing problem errorCode="+getErrorCode());
+	}
+
+	checkEncryptedContent();
+	internal_fetch = false;
+}
 
 void CXref::cleanUp()
 {
@@ -92,7 +114,7 @@ using namespace debug;
 	}
 	*/
 	
-	kernelPrintDbg(DBG_INFO, "Deallocating internal structures");
+	kernelPrintDbg(DBG_DBG, "Deallocating internal structures");
 	cleanUp();
 	
 	// XRef doesn't deallocate stream, so it has to be deallocated here
@@ -106,6 +128,8 @@ using namespace debug;
 	kernelPrintDbg(DBG_DBG, ref);
 	assert(ref.num!=0);
 
+	check_need_credentials(this);
+
 	// discards from cache
 	/* FIXME uncoment if cache is available
 	if(cache)
@@ -117,7 +141,7 @@ using namespace debug;
 	if(!clonedObject)
 	{
 		// cloning has failed
-		kernelPrintDbg(DBG_ERR, "Object can't be cloned.");
+		kernelPrintDbg(DBG_ERR, ref <<" object ("<< instance->getType()<<") can't be cloned.");
 		throw NotImplementedException("clone failure.");
 	}
 
@@ -163,22 +187,29 @@ using namespace debug;
 
 	kernelPrintDbg(DBG_DBG, "name="<<name<<" value type="<<value->getType());
 	
+	check_need_credentials(this);
+
 	Dict * trailer = trailerDict.getDict(); 
 
 	Object * clonedObject=value->clone();
 	if(!clonedObject)
 	{
 		// cloning has failed
-		kernelPrintDbg(DBG_ERR, "Object can't be cloned. Uses objNull instead");
+		kernelPrintDbg(DBG_ERR, "object ("
+				<< value->getType()
+				<<") can't be cloned for name="
+				<<name
+				<<". Uses objNull instead");
 		throw NotImplementedException("clone failure.");
 	}
-	char * key=strdup(name);
+	char * key=copyString(name);
 	::Object * prev = trailer->update(key, clonedObject);
+	gfree(clonedObject);
 
 	// update doesn't store key if key, value has been already in the 
 	// dictionary
 	if(prev)
-		free(key);
+		gfree(key);
 
 	return prev;
 }
@@ -192,6 +223,8 @@ using namespace debug;
 
 	kernelPrintDbg(DBG_DBG, "");
 	
+	check_need_credentials(this);
+
 	// goes through entries array in XRef class (xref entries)
 	// and reuses first free entry with its gen number.
 	// Considers just first XRef::getNumObjects because entries array
@@ -271,7 +304,7 @@ using namespace debug;
 	// initialized value is overwritten by change method.
 	::Ref objRef={num, gen};
 	newStorage.put(objRef, RESERVED_REF);
-	kernelPrintDbg(DBG_INFO, objRef<<" registered to newStorage");
+	kernelPrintDbg(DBG_DBG, objRef<<" registered to newStorage");
 
 	return objRef;
 }
@@ -346,6 +379,8 @@ using namespace debug;
 
 	kernelPrintDbg(DBG_DBG, "");
 
+	check_need_credentials(this);
+
 	// checks newly created object only with true flag
 	// not found returns 0, so it's ok
 	::RefState state;
@@ -368,6 +403,7 @@ using namespace debug;
 
 	if(!obj1 || !obj2)
 	{
+		assert(!obj1 || !obj2);
 		if(!obj1)
 			kernelPrintDbg(DBG_ERR, "obj1 is null");
 		
@@ -435,7 +471,9 @@ using namespace debug;
 	if(!retValue)
 	{
 		// cloning has failed
-		kernelPrintDbg(DBG_ERR, "Object can't be cloned. Uses objNull instead");
+		kernelPrintDbg(DBG_ERR, "Trailer::"<<name<<" ("
+				<<obj.getType() 
+				<<") can't be cloned. Uses objNull instead");
 		throw NotImplementedException("clone failure.");
 	}
 
@@ -447,6 +485,9 @@ using namespace debug;
 using namespace debug;
 
 	kernelPrintDbg(DBG_DBG, "");
+
+	// TODO needs credentials ?
+
 	// gets object
 	::Object docObj;
 	XRef::getDocInfo(&docObj);
@@ -458,7 +499,9 @@ using namespace debug;
 	if(!retObj)
 	{
 		// cloning has failed
-		kernelPrintDbg(DBG_ERR, "Object can't be cloned. Uses objNull instead");
+		kernelPrintDbg(DBG_ERR, "Trailer::Info ("
+				<<docObj.getType()
+				<<") can't be cloned. Uses objNull instead");
 		throw NotImplementedException("clone failure.");
 	}
 	*obj=*retObj;
@@ -471,6 +514,9 @@ using namespace debug;
 using namespace debug;
 
 	kernelPrintDbg(DBG_DBG, "");
+
+	// TODO needs credentials ?
+
 	// gets object
 	::Object docObj;
 	XRef::getDocInfoNF(&docObj);
@@ -483,7 +529,9 @@ using namespace debug;
 	if(!retObj)
 	{
 		// cloning has failed
-		kernelPrintDbg(DBG_ERR, "Object can't be cloned. Uses objNull instead");
+		kernelPrintDbg(DBG_ERR, "Trailer::Info ("
+				<<docObj.getType()
+				<<") can't be cloned. Uses objNull instead");
 		throw NotImplementedException("clone failure.");
 	}
 	// shallow copy of the content (deep copied)
@@ -498,6 +546,13 @@ using namespace debug;
 
 	kernelPrintDbg(DBG_DBG, "num="<<num<<" gen="<<gen);
 	
+	// internal objects fetching don't require credentials.
+	// However we have to be very carefull where to enable
+	// internal_fetch, because if we try to fetch encrypted
+	// content we will get wrong objects
+	if(!internal_fetch)
+		check_need_credentials(this);
+
 	::Ref ref={num, gen};
 	
 	// tries to use cache
@@ -512,7 +567,7 @@ using namespace debug;
 	ObjectEntry * entry=changedStorage.get(ref);
 	if(entry)
 	{
-		kernelPrintDbg(DBG_INFO, ref<<" is changed - using changedStorage");
+		kernelPrintDbg(DBG_DBG, ref<<" is changed - using changedStorage");
 		
 		// object has been changed
 		// this clone never fails, because it had to be cloned before storing to
@@ -522,10 +577,11 @@ using namespace debug;
 		{
 			// this shouldn't not happen
 			// TODO handle - exception should be called
-			kernelPrintDbg(DBG_CRIT, "Changed object is NULL!");
+			kernelPrintDbg(DBG_CRIT, ref << " changed object is NULL!");
 			return obj;
 		}
 		::Object * deepCopy=entry->object->clone();
+		assert(deepCopy);
 
 		// shallow copy of content
 		// content is deep copy of found object, so
@@ -538,9 +594,15 @@ using namespace debug;
 	}
 
 	// delegates to original implementation
-	kernelPrintDbg(DBG_INFO, ref<<" is not changed - using Xref");
+	kernelPrintDbg(DBG_DBG, ref<<" is not changed - using Xref");
 	Object tmpObj;
 	XRef::fetch(num, gen, &tmpObj);
+	if (!isOk())
+	{
+		kernelPrintDbg(DBG_ERR, ref<<" object fetching failed with code="
+				<<errCode);
+		throw MalformedFormatExeption("bad stream");
+	}
 
 	// clones fetched object
 	// this has to be done because return value may be stream and we want to
@@ -551,7 +613,9 @@ using namespace debug;
 	if(!cloneObj)
 	{
 		// cloning has failed
-		kernelPrintDbg(DBG_ERR, "Object can't be cloned. Uses objNull instead");
+		kernelPrintDbg(DBG_ERR, ref << " object ("
+				<<tmpObj.getType()
+				<<") can't be cloned. Uses objNull instead");
 		throw NotImplementedException("clone failure.");
 	}
 
@@ -585,28 +649,93 @@ using namespace debug;
 		if(i->second==INITIALIZED_REF)
 			++newSize;
 
-	kernelPrintDbg(DBG_INFO, "original objects count="<<XRef::getNumObjects()<<" newly created="<<newSize);
+	kernelPrintDbg(DBG_DBG, "original objects count="<<XRef::getNumObjects()<<" newly created="<<newSize);
 	return XRef::getNumObjects() + newSize;
 }
 
 
-void CXref::reopen(size_t xrefOff)
+void CXref::reopen(size_t xrefOff, bool dropChanges)
 {
 using namespace debug;
 
 	kernelPrintDbg(DBG_DBG, "");
 
 	// clears all object storages
-	kernelPrintDbg(DBG_INFO, "Destroying CXref internals");
-	cleanUp();
+	kernelPrintDbg(DBG_DBG, "Destroying CXref internals");
+	if(dropChanges)
+		cleanUp();
 
 	// clears XRef internals and forces to fill them again
-	kernelPrintDbg(DBG_INFO, "Destroing XRef internals");
+	kernelPrintDbg(DBG_DBG, "Destroing XRef internals");
 	XRef::destroyInternals();
-	kernelPrintDbg(DBG_INFO, "Initializes XRef internals");
+	kernelPrintDbg(DBG_DBG, "Initializes XRef internals");
 	XRef::initInternals(xrefOff);
 
 	// sets lastXRefPos to xrefOff, because initRevisionSpecific doesn't do it
 	lastXRefPos=xrefOff;
 	kernelPrintDbg(DBG_DBG, "New lastXRefPos value="<<lastXRefPos);
+
+	// checks encryption state for the revision
+	checkEncryptedContent();
+}
+
+bool CXref::checkEncryptedContent()
+{
+	Object encrypt;
+	needs_credentials = false;
+	enableInternalFetch();
+	getTrailerDict()->dictLookup("Encrypt", &encrypt);
+	disableInternalFetch();
+	encrypted = false;
+	if ((encrypted = encrypt.isDict())) {
+		needs_credentials = true;
+		encrypted = true;
+	}
+	encrypt.free();
+
+	return needs_credentials;
+}
+
+void CXref::setCredentials(const char * ownerPasswd, const char * userPasswd)
+{
+	if(!needs_credentials)
+	{
+		kernelPrintDbg(debug::DBG_WARN, "credentials provided when not required");
+		return;
+	}
+
+	GString * op = NULL, * up = NULL;
+	if(ownerPasswd)
+		op = new GString(ownerPasswd);
+	if(userPasswd)
+		up = new GString(userPasswd);
+
+	GBool result;
+	enableInternalFetch();
+	SecurityHandler * handler = ::checkEncryptionCred(this, op, up, result);
+	disableInternalFetch();
+
+	// we are not able to find a security handler
+	if(!result && !handler)
+	{
+		kernelPrintDbg(debug::DBG_ERR, "Unable to find proper security handler");
+		throw NotImplementedException("security handler");
+	}
+
+	// authorization failed
+	if(!result && handler)
+	{
+		kernelPrintDbg(debug::DBG_ERR, "Bad credentials");
+		delete handler;
+		throw PermissionException("Bad credentials");
+	}
+
+	needs_credentials = false;
+	if(handler)
+	{
+		kernelPrintDbg(debug::DBG_DBG, "Setting provided ecnryption credentials.");
+		::setEncryptionCred(this, handler);
+		delete handler;
+	}else
+		kernelPrintDbg(debug::DBG_DBG, "No special credentials required for encrypted document");
 }
