@@ -1,9 +1,9 @@
 /*
  * PDFedit - free program for PDF document manipulation.
- * Copyright (C) 2006, 2007, 2008  PDFedit team: Michal Hocko,
- *                                              Miroslav Jahoda,
- *                                              Jozef Misutka,
- *                                              Martin Petricek
+ * Copyright (C) 2006-2009  PDFedit team: Michal Hocko,
+ *                                        Jozef Misutka,
+ *                                        Martin Petricek
+ *                   Former team members: Miroslav Jahoda
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,15 +34,17 @@
 
 #include "kernel/static.h"
 #include "kernel/cxref.h"
-#include "kernel/streamwriter.h"
-#include "kernel/pdfwriter.h"
 
+class StreamWriter;
 
 namespace pdfobjects
 {
+class CPdf;
+struct IndiRef;
+class XRefWriter;
 
-namespace utils
-{
+namespace utils {
+class IPdfWriter;
 
 /** Checks whether given stream is linearized.
  * @param stream Pdf stream to read (from the file begin).
@@ -53,11 +55,20 @@ namespace utils
  * dictionary, checks whether it contains Linearized version entry. If so
  * returns true and if given ref is not NULL, sets object and generation number.
  * Otherwise just returns false and doesn't care for ref parameter.
+ * <br>
+ * This method doesn't check whether credentials are set properly. You have
+ * to do it before it is called.
  *
  * @return true if first indirect object is Linearized dictionary, false
  * otherwise.
  */
-bool checkLinearized(StreamWriter & stream, XRef * xref, Ref * ref);
+bool checkLinearized(StreamWriter & stream, CXref * xref, Ref * ref);
+
+/** Checks whether the current revision is the most recent one.
+ * @param xref Xref.
+ * @return true if the current revision is the latest one, false otherwise.
+ */
+bool isLatestRevision(const XRefWriter &xref);
 
 } // end of namespace utils
 
@@ -71,7 +82,8 @@ bool checkLinearized(StreamWriter & stream, XRef * xref, Ref * ref);
  * <li> controling checking routines - mode field controls level of checking.
  *	(getMode, setMode methods).
  * <li> manipulating revisions of document. Revisions are available through
- *	their numbers (0 for newest and getRevisionCount-1 for latest). 
+ *	their numbers (0 for the oldest and getRevisionCount-1 for the most recent 
+ *	one - use isLatestRevision helper function for testing). 
  *	Changes can be made only in newest revision. Older revisions are only 
  *	read-only. (getRevisionCount, getActualRevision, changeRevision 
  *	methods).
@@ -214,6 +226,8 @@ protected:
 	 * Parses Trailer dictionary and gets Prev field value. If not present,
 	 * assumes no more revisions are available. Otherwise stores that position
 	 * to revisions storage as later revision and continues same way.
+	 * <br>
+	 * Sets revision field to the most recent one as a side effect.
 	 */
 	void collectRevisions();
 
@@ -257,12 +271,7 @@ public:
 	 *
 	 * Deallocates pdfWriter field if it is non NULL.
 	 */
-	~XRefWriter()
-	{
-		kernelPrintDbg(debug::DBG_DBG, "");
-		if(pdfWriter)
-			delete pdfWriter;
-	}
+	~XRefWriter();
 	
 	/** Sets new pdf writer implementator.
 	 * @param writer Implementation of IPdfWriter (must be non NULL).
@@ -337,6 +346,7 @@ public:
 	 * revision is not the newest one or if pdf is in read-only mode.
 	 * @throw ElementBadTypeException if mode is paranoid and paranoidCheck
 	 * method fails for obj.
+	 * @throw NotImplementedException if document is encrypted.
 	 */ 
 	void changeObject(int num, int gen, ::Object * obj);
 
@@ -354,6 +364,8 @@ public:
 	 * 
 	 * @throw ReadOnlyDocumentException if no changes can be done because actual
 	 * revision is not the newest one or if pdf is in read-only mode.
+	 * @throw NotImplementedException if document is encrypted or when trailer
+	 * dictionary can't be cloned (because clone method failes).
 	 * @return Previous value of object or 0 if previous revision not
 	 * available (new name value pair in trailer).
 	 */
@@ -402,12 +414,11 @@ public:
 	/** Changes revision of document.
 	 * @param revNumber Number of the revision.
 	 *
-	 * Jumps to the given revision. 0 stands for newest revision, older 
-	 * have higher number. The oldest revision has getRevisionCount()-1
-	 * number.
+	 * Jumps to the given revision. 0 stands for the oldest revision while
+	 * newer revisions have higher number. 
 	 * <p>
-	 * XRefWriter doesn't enable to make changes if revision is not 0 (most
-	 * actual). This means that all methods producing changes are invalid 
+	 * XRefWriter doesn't enable to make changes if revision is not the most
+	 * recent one. This means that all methods producing changes are invalid 
 	 * and produces error when called.
 	 * <br>
 	 * This is because branching is not implementable in PDF structure.
@@ -489,12 +500,12 @@ public:
 	 */ 
 	virtual ::Object * fetch(int num, int gen, ::Object * obj)
 	{
-		// newest revision may contain changes, so uses
+		// the newest revision may contain changes, so uses
 		// CXref implementation
-		if(!revision)
+		if(utils::isLatestRevision(*this))
 			return CXref::fetch(num, gen, obj);
 
-		// we are in later revision, we have to use only XRef
+		// we are in an older revision, we have to use only XRef
 		// implementation
 		return XRef::fetch(num, gen, obj);
 	}
@@ -502,8 +513,9 @@ public:
 	/** Checks if given reference is known.
 	 * @param ref Reference to check.
 	 *
-	 * Checks if revision is 0 (the newest one) delegates to CXref
-	 * implementation, because there can be also new referencies.
+	 * Checks if the current revision is the newest one and 
+	 * delegates to CXref implementation if this is true, because 
+	 * there can be also new referencies.
 	 * Otherwise searches only in XRef::entries (only referencies
 	 * from document).
 	 *
@@ -513,7 +525,7 @@ public:
 	virtual RefState knowsRef(::Ref& ref)
 	{
 		// if we are in newest revision, delegates to CXref
-		if(!revision)
+		if(utils::isLatestRevision(*this))
 			return CXref::knowsRef(ref);
 				
 		// otherwise use XRef directly
@@ -528,20 +540,17 @@ public:
 	 * @see RefState
 	 * @return Reference state.
 	 */
-	virtual RefState knowsRef(IndiRef ref)
-	{
-		::Ref xpdfRef={ref.num, ref.gen};
-		// otherwise use XRef directly
-		return knowsRef(xpdfRef);
-	}
-
+	virtual RefState knowsRef(IndiRef& ref);
+	
 	/** Registers new reference.
 	 *
 	 * This is just wrapper for CXref::reference method.
 	 * Only thing which is done here is that revision field
-	 * is checked and if revision is 0 (most recent), delegates 
-	 * to the to the CXref::reserveRef. Otherwise deny to create, 
-	 * because it is not possible to do changes to a older release.
+	 * is checked and if the current revision is the most recent one, 
+	 * delegates to the to the CXref::reserveRef. 
+	 *
+	 * Otherwise deny to create, because it is not possible to do changes 
+	 * to a older release.
 	 *
 	 * @throw ReadOnlyDocumentException if no changes can be done because actual
 	 * revision is not the newest one or if pdf is in read-only mode.
@@ -554,12 +563,13 @@ public:
 	 *
 	 * This is just wrapper for CXref::createObject method.
 	 * Only thing which is done here is that revision field
-	 * is checked and if revision is 0 (most recent), delegates 
+	 * is checked and if the current revision is the most recent one, delegates 
 	 * to the to the CXref::createObject. Otherwise deny to make create, 
 	 * because it is not possible to do changes to a older release.
 	 *
 	 * @throw ReadOnlyDocumentException if no changes can be done because actual
 	 * revision is not the newest one or if pdf is in read-only mode.
+	 * @throw NotImplementedException if document is encrypted.
 	 */
 	virtual ::Object * createObject(::ObjType type, ::Ref * ref);
 	
