@@ -1,9 +1,9 @@
 /*
  * PDFedit - free program for PDF document manipulation.
- * Copyright (C) 2006, 2007, 2008  PDFedit team: Michal Hocko,
- *                                              Miroslav Jahoda,
- *                                              Jozef Misutka,
- *                                              Martin Petricek
+ * Copyright (C) 2006-2009  PDFedit team: Michal Hocko,
+ *                                        Jozef Misutka,
+ *                                        Martin Petricek
+ *                   Former team members: Miroslav Jahoda
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,15 +40,15 @@ using namespace boost;
 //=====================================================================================
 
 bool
-setCS (__attribute__((unused))	ostream& oss, const char* fileName)
+setCS (UNUSED_PARAM	ostream& oss, const char* fileName)
 {
-	boost::shared_ptr<CPdf> ppdf (getTestCPdf (fileName), pdf_deleter());
+	boost::shared_ptr<CPdf> ppdf = getTestCPdf (fileName);
 	size_t pagecnt = ppdf->getPageCount ();
 	ppdf.reset();
 	
 	for (size_t i = 0; i < pagecnt && i < TEST_MAX_PAGE_COUNT; ++i)
 	{
-		boost::shared_ptr<CPdf> pdf (getTestCPdf (fileName), pdf_deleter());
+		boost::shared_ptr<CPdf> pdf = getTestCPdf (fileName);
 		boost::shared_ptr<CPage> page = pdf->getPage (i+1);
 		vector<boost::shared_ptr<CContentStream> > ccs;
 		page->getContentStreams (ccs);
@@ -106,18 +106,15 @@ setCS (__attribute__((unused))	ostream& oss, const char* fileName)
 bool
 frontinsert (ostream& oss, const char* fileName)
 {
-	boost::shared_ptr<CPdf> ppdf (getTestCPdf (fileName), pdf_deleter());
+	boost::shared_ptr<CPdf> ppdf = getTestCPdf (fileName);
 	size_t pagecnt = ppdf->getPageCount ();
 	ppdf.reset();
 	
 	for (size_t i = 0; i < pagecnt && i < TEST_MAX_PAGE_COUNT; ++i)
 	{
-		boost::shared_ptr<CPdf> pdf (getTestCPdf (fileName), pdf_deleter());
+		boost::shared_ptr<CPdf> pdf = getTestCPdf (fileName);
 		boost::shared_ptr<CPage> page = pdf->getPage (i + 1);
 		
-		// parse the content stream
-		page->parseContentStream ();
-	
 		vector<boost::shared_ptr<CContentStream> > ccs;
 		page->getContentStreams (ccs);
 		assert (!ccs.empty());
@@ -148,18 +145,15 @@ frontinsert (ostream& oss, const char* fileName)
 bool
 position (ostream& oss, const char* fileName, const libs::Rectangle rc)
 {
-	boost::shared_ptr<CPdf> ppdf (getTestCPdf (fileName), pdf_deleter());
+	boost::shared_ptr<CPdf> ppdf = getTestCPdf (fileName);
 	size_t pagecnt = ppdf->getPageCount ();
 	ppdf.reset();
 	
 	for (size_t i = 0; i < pagecnt && i < TEST_MAX_PAGE_COUNT; ++i)
 	{
-		boost::shared_ptr<CPdf> pdf (getTestCPdf (fileName), pdf_deleter());
+		boost::shared_ptr<CPdf> pdf = getTestCPdf (fileName);
 		boost::shared_ptr<CPage> page = pdf->getPage (i + 1);
 		
-		// parse the content stream
-		page->parseContentStream ();
-		
 		std::vector<shared_ptr<PdfOperator> > ops;
 		page->getObjectsAtPosition (ops, rc);
 
@@ -193,7 +187,7 @@ namespace  {
 		  dict.initDict (xref);
 		  o.free ();
 		  
-		  parser->getObj(&o);
+		  parser->getObj(&o); // FIXME handle return value
 		  while (!o.isCmd("ID") && !o.isEOF()) 
 		  {
 			if (!o.isName()) 
@@ -205,7 +199,7 @@ namespace  {
 			{
 			  char* key = copyString(o.getName());
 			  o.free();
-			  parser->getObj(&o);
+			  parser->getObj(&o); // FIXME handle return value
 			  if (o.isEOF() || o.isError()) 
 			  {
 				gfree (key);
@@ -214,7 +208,7 @@ namespace  {
 			  dict.dictAdd(key, &o);
 			}
 			
-			parser->getObj(&o);
+			parser->getObj(&o); // FIXME handle return value
 		  }
 		 
 		  
@@ -307,15 +301,15 @@ namespace  {
 } // namespace
 
 bool
-opcount (__attribute__((unused))	ostream& oss, const char* fileName)
+opcount (UNUSED_PARAM	ostream& oss, const char* fileName)
 {
-	boost::shared_ptr<CPdf> pdf (getTestCPdf (fileName), pdf_deleter());
+	boost::shared_ptr<CPdf> pdf = getTestCPdf (fileName);
 	/// Intermezzo
 	boost::scoped_ptr<PDFDoc> doc  (new PDFDoc (new GString(fileName), NULL, NULL));
 
 	for (size_t i = 0; i < pdf->getPageCount() && i < TEST_MAX_PAGE_COUNT; ++i)
 	{
-		boost::shared_ptr<CPdf> pdf (getTestCPdf (fileName), pdf_deleter());
+		boost::shared_ptr<CPdf> pdf = getTestCPdf (fileName);
 		boost::shared_ptr<CPage> page = pdf->getPage (i + 1);
 
 		int pagesNum = i + 1;
@@ -344,7 +338,7 @@ opcount (__attribute__((unused))	ostream& oss, const char* fileName)
 			if (isStream (tmp) && obj.isStream())
 			{
 				
-				streams.push_back ( shared_ptr<CStream> (new CStream (*pdf, obj, tmp->getIndiRef())));
+				streams.push_back ( shared_ptr<CStream> (new CStream (pdf, obj, tmp->getIndiRef())));
 
 			}else if (isArray (tmp) && obj.isArray())
 			{
@@ -352,7 +346,8 @@ opcount (__attribute__((unused))	ostream& oss, const char* fileName)
 				for (int i = 0; i < obj.arrayGetLength(); ++i)
 				{
 					shared_ptr<IProperty> ent = utils::getReferencedObject(IProperty::getSmartCObjectPtr<CArray> (tmp)->getProperty (i));
-					streams.push_back ( shared_ptr<CStream> (new CStream (*pdf,  *(obj.arrayGet (i, &o)), ent->getIndiRef())));
+					streams.push_back ( shared_ptr<CStream> (new CStream (pdf,  *(obj.arrayGet (i, &o)), ent->getIndiRef())));
+					o.free ();
 				}
 			}else
 			{
@@ -407,9 +402,9 @@ opcount (__attribute__((unused))	ostream& oss, const char* fileName)
 //=====================================================================================
 
 bool
-printContentStream (__attribute__((unused))	ostream& oss, const char* fileName)
+printContentStream (UNUSED_PARAM	ostream& oss, const char* fileName)
 {
-	boost::shared_ptr<CPdf> pdf (getTestCPdf (fileName), pdf_deleter());
+	boost::shared_ptr<CPdf> pdf = getTestCPdf (fileName);
 	for (size_t i = 0; i < pdf->getPageCount() && i < TEST_MAX_PAGE_COUNT; ++i)
 	{
 		boost::shared_ptr<CPage> page = pdf->getPage (i + 1);
@@ -437,9 +432,9 @@ printContentStream (__attribute__((unused))	ostream& oss, const char* fileName)
 //=========================================================================
 
 bool
-addcc (__attribute__((unused))	ostream& oss, const char* fileName)
+addcc (UNUSED_PARAM	ostream& oss, const char* fileName)
 {
-	boost::shared_ptr<CPdf> pdf (getTestCPdf (fileName), pdf_deleter());
+	boost::shared_ptr<CPdf> pdf = getTestCPdf (fileName);
 	for (size_t i = 0; i < pdf->getPageCount() && i < TEST_MAX_PAGE_COUNT; ++i)
 	{
 		boost::shared_ptr<CPage> page = pdf->getPage (i + 1);
@@ -455,7 +450,7 @@ addcc (__attribute__((unused))	ostream& oss, const char* fileName)
 			PdfOperator::Operands operands;
 				
 			Opers ops;
-			ops.push_back (boost::shared_ptr<PdfOperator> (new SimpleGenericOperator("lala",0,operands)));
+			ops.push_back (createOperator("lala",operands));
 			page->addContentStreamToFront (ops);
 
 			vector<boost::shared_ptr<CContentStream> > cccs;
@@ -487,13 +482,13 @@ addcc (__attribute__((unused))	ostream& oss, const char* fileName)
 //=========================================================================
 
 bool
-settm (__attribute__((unused))	ostream& oss, const char* fileName)
+settm (UNUSED_PARAM	ostream& oss, const char* fileName)
 {
-	boost::shared_ptr<CPdf> ppdf (getTestCPdf (fileName), pdf_deleter());
+	boost::shared_ptr<CPdf> ppdf = getTestCPdf (fileName);
 	size_t num = ppdf->getPageCount ();
 	for (size_t i = 0; i < num && i < TEST_MAX_PAGE_COUNT; ++i)
 	{
-		boost::shared_ptr<CPdf> pdf (getTestCPdf (fileName), pdf_deleter());
+		boost::shared_ptr<CPdf> pdf = getTestCPdf (fileName);
 		boost::shared_ptr<CPage> page = pdf->getPage (i + 1);
 
 		// Print content stream
@@ -525,10 +520,10 @@ settm (__attribute__((unused))	ostream& oss, const char* fileName)
 
 
 bool
-primitiveprintContentStream (__attribute__((unused))	ostream& oss, const char* fileName)
+primitiveprintContentStream (UNUSED_PARAM	ostream& oss, const char* fileName)
 {
 	{// what if file is corrupted etc..
-		boost::shared_ptr<CPdf> pdf (getTestCPdf (fileName), pdf_deleter());
+		boost::shared_ptr<CPdf> pdf = getTestCPdf (fileName);
 	}
 	boost::scoped_ptr<PDFDoc> doc (new PDFDoc (new GString(fileName), NULL, NULL));
 	int pagesNum = 1;
@@ -553,7 +548,7 @@ primitiveprintContentStream (__attribute__((unused))	ostream& oss, const char* f
 			);
 	
 	Object o;
-	parser->getObj (&o);
+	parser->getObj (&o); // FIXME handle return value
 	int i = 0;
 	
 	while (!o.isEOF()) 
@@ -563,7 +558,7 @@ primitiveprintContentStream (__attribute__((unused))	ostream& oss, const char* f
 		{
 			if (!img (parser.get(),o,xref))
 				return false;
-	    	parser->getObj(&o);
+	    	parser->getObj(&o); // FIXME handle return value
 		}
 		
 		if (o.isCmd ())
@@ -580,7 +575,7 @@ primitiveprintContentStream (__attribute__((unused))	ostream& oss, const char* f
 
 		// grab the next object
 		o.free ();
-		parser->getObj(&o);
+		parser->getObj(&o); // FIXME handle return value
 	}
 
 	obj.free ();
@@ -591,14 +586,14 @@ primitiveprintContentStream (__attribute__((unused))	ostream& oss, const char* f
 //=========================================================================
 
 bool
-cstreamsreader (__attribute__((unused))	ostream& oss, const char* fileName)
+cstreamsreader (UNUSED_PARAM	ostream& oss, const char* fileName)
 {
-	boost::shared_ptr<CPdf> pdf (getTestCPdf (fileName), pdf_deleter());
+	boost::shared_ptr<CPdf> pdf = getTestCPdf (fileName);
 	if (1 > pdf->getPageCount())
 		return true;
 
 	
-	boost::shared_ptr<CPdf> ppdf (getTestCPdf (fileName), pdf_deleter());
+	boost::shared_ptr<CPdf> ppdf = getTestCPdf (fileName);
 	size_t num = ppdf->getPageCount ();
 	for (size_t i = 0; i < num && i < TEST_MAX_PAGE_COUNT; ++i)
 	{
@@ -606,7 +601,7 @@ cstreamsreader (__attribute__((unused))	ostream& oss, const char* fileName)
 		// - init our reader then xpdf reader
 		// - read one object and compare it
 		//
-		boost::shared_ptr<CPdf> pdf (getTestCPdf (fileName), pdf_deleter());
+		boost::shared_ptr<CPdf> pdf = getTestCPdf (fileName);
 		boost::shared_ptr<CPage> page = pdf->getPage (i + 1);
 
 		/// Intermezzo cstreamsreader
@@ -635,7 +630,7 @@ cstreamsreader (__attribute__((unused))	ostream& oss, const char* fileName)
 			if (isStream (tmp) && obj.isStream())
 			{
 				
-				streams.push_back ( shared_ptr<CStream> (new CStream (*pdf, obj, tmp->getIndiRef())));
+				streams.push_back ( shared_ptr<CStream> (new CStream (pdf, obj, tmp->getIndiRef())));
 
 			}else if (isArray (tmp) && obj.isArray())
 			{
@@ -643,7 +638,8 @@ cstreamsreader (__attribute__((unused))	ostream& oss, const char* fileName)
 				for (int i = 0; i < obj.arrayGetLength(); ++i)
 				{
 					shared_ptr<IProperty> ent = utils::getReferencedObject (IProperty::getSmartCObjectPtr<CArray> (tmp)->getProperty (i));
-					streams.push_back ( shared_ptr<CStream> (new CStream (*pdf,  *(obj.arrayGet (i, &o)), ent->getIndiRef())));
+					streams.push_back ( shared_ptr<CStream> (new CStream (pdf,  *(obj.arrayGet (i, &o)), ent->getIndiRef())));
+					o.free ();
 				}
 			}else
 			{
@@ -664,7 +660,7 @@ cstreamsreader (__attribute__((unused))	ostream& oss, const char* fileName)
 					)
 				);
 		Object o1;
-		parser->getObj (&o1);
+		parser->getObj (&o1); // FIXME handle return value
 		///
 		obj.free ();
 
@@ -686,7 +682,7 @@ cstreamsreader (__attribute__((unused))	ostream& oss, const char* fileName)
 			reader.getXpdfObject (o);
 		
 			o1.free();
-			parser->getObj (&o1);
+			parser->getObj (&o1); // FIXME handle return value
 
 		}
 		reader.close ();
@@ -770,13 +766,17 @@ public:
 		{
 			OUTPUT << "Testing filename: " << *it << endl;
 			
-			TEST(" test matrix");
-			CPPUNIT_ASSERT (settm (OUTPUT, (*it).c_str()));
-			OK_TEST;
+			BEGIN_CHECK_READONLY;
+				TEST(" test matrix");
+				CPPUNIT_ASSERT (settm (OUTPUT, (*it).c_str()));
+				OK_TEST;
+			END_CHECK_READONLY;
 
-			TEST(" add content stream");
-			CPPUNIT_ASSERT (addcc (OUTPUT, (*it).c_str()));
-			OK_TEST;
+			BEGIN_CHECK_READONLY;
+				TEST(" add content stream");
+				CPPUNIT_ASSERT (addcc (OUTPUT, (*it).c_str()));
+				OK_TEST;
+			END_CHECK_READONLY;
 		}
 	}
 	//
@@ -792,13 +792,15 @@ public:
 		{
 			OUTPUT << "Testing filename: " << *it << endl;
 			
-			TEST(" insert at front");
-			CPPUNIT_ASSERT (frontinsert (OUTPUT, (*it).c_str()));
-			OK_TEST;
+			BEGIN_CHECK_READONLY;
+				TEST(" insert at front");
+				CPPUNIT_ASSERT (frontinsert (OUTPUT, (*it).c_str()));
+				OK_TEST;
 
-			TEST(" add content stream");
-			CPPUNIT_ASSERT (addcc (OUTPUT, (*it).c_str()));
-			OK_TEST;
+				TEST(" add content stream");
+				CPPUNIT_ASSERT (addcc (OUTPUT, (*it).c_str()));
+				OK_TEST;
+			END_CHECK_READONLY;
 		}
 	}
 
@@ -886,9 +888,11 @@ public:
 		{
 			OUTPUT << "Testing filename: " << *it << endl;
 
-			TEST(" setCS");
-			CPPUNIT_ASSERT (setCS (OUTPUT, (*it).c_str()));
-			OK_TEST;
+			BEGIN_CHECK_READONLY;
+				TEST(" setCS");
+				CPPUNIT_ASSERT (setCS (OUTPUT, (*it).c_str()));
+				OK_TEST;
+			END_CHECK_READONLY;
 		}
 	}
 
