@@ -5,14 +5,16 @@
 // Copyright 1996-2003 Glyph & Cog, LLC
 //
 // Changes: 
-// Michal Hocko   - public clone method for deep copy of Array
+// Michal Hocko   
+// 	- public clone method for deep copy of Array
+// 	- getXRef added
 //
 //========================================================================
 
 #ifndef ARRAY_H
 #define ARRAY_H
 
-#include <aconf.h>
+#include <xpdf-aconf.h>
 
 #ifdef USE_GCC_PRAGMAS
 #pragma interface
@@ -51,6 +53,8 @@ public:
   Object *get(int i, Object *obj);
   Object *getNF(int i, Object *obj);
 
+  XRef *getXRef() {return xref;}
+
 private:
 
   XRef *xref;			// the xref table for this PDF file
