@@ -9,7 +9,7 @@
 //
 //========================================================================
 
-#include <aconf.h>
+#include <xpdf-aconf.h>
 
 #ifdef USE_GCC_PRAGMAS
 #pragma implementation
@@ -90,7 +90,7 @@ Object * Object::clone()const
            goto cloneerror;
          break;
       case objName:
-         result->name=strdup(name);
+         result->name=copyString(name);
          if(!result->name)
            goto cloneerror;
          break;
@@ -113,7 +113,7 @@ Object * Object::clone()const
          result->ref=ref;
          break;
       case objCmd:
-         result->cmd=strdup(cmd);
+         result->cmd=copyString(cmd);
          if(!result->cmd)
            goto cloneerror;
          break;
@@ -141,6 +141,13 @@ Object *Object::initArray(XRef *xref) {
   return this;
 }
 
+Object *Object::initArray(Array *array) {
+  initObj(objArray);
+  this->array = array;
+  array->incRef();
+  return this;
+}
+
 Object *Object::initDict(XRef *xref) {
   initObj(objDict);
   dict = new Dict(xref);
